from manim import *
import random
from manima.mobjects import MediaMobject, HandDrawnArrow


class Firewall(VGroup):
    def __init__(
        self,
        num_bricks_x=4,
        num_bricks_y=6,
        brick_width=1,
        brick_height=0.5,
        brick_spacing_x=0.1,
        brick_spacing_y=0.1,
        corner_radius=0.05,
        fill_color=BLUE,
        outline_color=WHITE,
        rotation_x=0,  # X轴旋转
        rotation_y=0,  # Y轴旋转
        rotation_z=0,  # Z轴旋转
        **kwargs
    ):
        super().__init__(**kwargs)
        self.fill_color = fill_color
        self.outline_color = outline_color

        # Create a list of all possible brick positions
        positions = [(i, j) for j in range(num_bricks_y)
                     for i in range(num_bricks_x)]

        # Shuffle the positions to randomize brick creation order
        random.shuffle(positions)

        for i, j in positions:
            text = f"{i},{j}"
            brick_group = self.create_brick(
                brick_width, brick_height, corner_radius, text)
            brick_group.shift(
                RIGHT * i * (brick_width + brick_spacing_x) +
                UP * j * (brick_height + brick_spacing_y)
            )
            self.add(brick_group)

        self.center()
        self.rotate(rotation_x, axis=RIGHT)  # 绕X轴旋转
        self.rotate(rotation_y, axis=UP)     # 绕Y轴旋转
        self.rotate(rotation_z, axis=OUT)    # 绕Z轴旋转

    def create_brick(self, brick_width, brick_height, corner_radius, text):
        brick_group = VGroup()

        brick = RoundedRectangle(
            width=brick_width,
            height=brick_height,
            corner_radius=corner_radius,
            fill_color=self.fill_color,
            fill_opacity=1,
            stroke_color=self.outline_color,
            stroke_width=2
        )

        number = Text(text, font_size=24, color=WHITE)
        number.set_z_index(1)
        number.move_to(brick.get_center())

        brick_group.add(brick, number)
        return brick_group

    def set_color(self, color):
        for brick_group in self.submobjects:
            brick_group[0].set_fill(color)
        return self

    def set_text(self, text):
        for brick_group in self.submobjects:
            brick_group[1] = Text(text, font_size=24, color=WHITE)
            brick_group[1].move_to(brick_group[0].get_center())
        return self


def create_red_cross() -> VGroup:
    cross = VGroup()
    line1 = Line(UP + LEFT, DOWN + RIGHT, color=RED, stroke_width=6)
    line2 = Line(UP + RIGHT, DOWN + LEFT, color=RED, stroke_width=6)
    cross.add(line1, line2)
    cross.scale(0.5)
    return cross


class FirewallExample(Scene):
    def construct(self):
        firewall = Firewall(
            brick_width=1.2,
            num_bricks_x=6,
            num_bricks_y=3,
            fill_color=BLUE,
        ).scale(0.8)
        cnurl = 'https://res.cloudinary.com/dz3h2cqfd/image/fetch/f_auto,q_auto:good,e_sharpen/e_contrast,co_rgb:000000/w_800/https://masbvijroflqdbrjtolx.supabase.co/storage/v1/object/public/Icons/icons/b92690b6c5cc654e3d7c52fdfe2e67e704e758434ad8386ca507897270c4cc19'
        cn = MediaMobject(cnurl).scale(0.8).to_corner(UL).set_color(GRAY)

        googleurl = 'https://res.cloudinary.com/dz3h2cqfd/image/fetch/f_auto,q_auto:good,e_sharpen/e_contrast,co_rgb:000000/w_800/https://masbvijroflqdbrjtolx.supabase.co/storage/v1/object/public/Icons/icons/ce2b79332d77198978ecee352b8cdf247e218b09e4d2722ac470e6cde037f008'
        google = MediaMobject(googleurl).scale(
            0.5).to_corner(DR)

        self.play(Create(firewall),
                  FadeIn(cn),
                  FadeIn(google))
        self.wait()

        self.play(firewall.animate.set_color(RED).scale(1.2).set_text("GFW"),
                  cn.animate.scale(0.8),
                  google.animate.scale(0.8))

        self.play(firewall.animate.rotate(
            90 * DEGREES, axis=UP),
            cn.animate.move_to(ORIGIN).to_edge(LEFT),
            google.animate.move_to(ORIGIN).to_edge(RIGHT))
        self.wait()

        cn2firewall = HandDrawnArrow(
            cn.get_center(), firewall.get_center()).set_color(RED)
        cross = create_red_cross()
        cross.move_to((cn.get_center()+firewall.get_center())/2)
        self.play(firewall.animate.rotate(
            30 * DEGREES, axis=LEFT).rotate(
            20 * DEGREES, axis=DOWN),
            cn.animate.set_color(RED),
            GrowFromPoint(cn2firewall, firewall.get_center()),
            FadeIn(cross))
        self.wait(2)
